package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.enums.Platform;
import com.qmth.sop.business.bean.auth.AuthBean;
import com.qmth.sop.business.bean.auth.ExpireTimeBean;
import com.qmth.sop.business.bean.result.LoginResult;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBSession;
import com.qmth.sop.business.mapper.SysUserMapper;
import com.qmth.sop.business.service.SysUserService;
import com.qmth.sop.business.service.TBSessionService;
import com.qmth.sop.business.util.AuthUtil;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.AppSourceEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.util.ServletUtil;
import com.qmth.sop.common.util.SessionUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-07-17
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    TBSessionService tbSessionService;

    @Override
    public LoginResult login(String password, SysUser sysUser, AppSourceEnum appSource) throws NoSuchAlgorithmException {
        //停用
        if (!sysUser.getEnable()) {
            throw ExceptionResultEnum.USER_ENABLE.exception();
        }

        Platform platform = ServletUtil.getRequestPlatform();
        String deviceId = ServletUtil.getRequestDeviceId();
        //添加用户鉴权缓存
        AuthBean authBean = commonCacheService.userAuthCache(sysUser.getId());
        Optional.ofNullable(authBean).orElseThrow(() -> ExceptionResultEnum.ROLE_ENABLE_AUTHORIZATION.exception());

        //生成token
        String token = SystemConstant.getNanoId();
        commonCacheService.userCache(sysUser.getId());
        //添加用户会话缓存
        Set<String> roleNamesSet = new HashSet<>(), roleTypes = new HashSet<>();
        List<String> roleNames = new ArrayList<>();
        roleNames.addAll(roleNamesSet);
        Collections.sort(roleNames);

        String sessionId = SessionUtil.digest(sysUser.getId(), Math.abs(roleNames.toString().hashCode()), platform.name());
        ExpireTimeBean expireTime = AuthUtil.getExpireTime(platform);
        TBSession tbSession = new TBSession(sessionId, String.valueOf(sysUser.getId()), roleNames.toString(),
                platform.name(), platform.name(), deviceId, ServletUtil.getRequest().getLocalAddr(), token,
                expireTime.getDate().getTime(), appSource);
        tbSessionService.saveOrUpdate(tbSession);
        commonCacheService.setUserSession(sessionId, tbSession);

        LoginResult loginResult = new LoginResult(sysUser, sessionId, token, Objects.nonNull(roleTypes) && roleTypes.size() > 0 ? roleTypes : roleNamesSet, appSource, SystemConstant.VERSION_VALUE);
        loginResult.setOrgInfo(Objects.nonNull(authBean.getOrg()) ? loginResult.new OrgNativeBean(authBean.getOrg()) : null);
        loginResult.setTime(System.currentTimeMillis());
        return loginResult;
    }
}
